# $Id:$

from math import log   # Natural logarithm

class BoundInteriorPointFramework:

    def __init__( self, nlp, **kwargs ):
        self.nlp = nlp
        self.mu = kwargs.get( 'mu', 1.0 )
        return

    # Evaluate the barrier function
    def Barrier( self, x, **kwargs ):
        mu = kwargs.get( 'mu', self.mu )

        barrier = self.nlp.obj( x )
        for i in self.nlp.lowerB + self.nlp.rangeB:
            barrier -= mu * log( x[i] - self.nlp.Lvar[i] )
        for i in self.nlp.upperB + self.nlp.rangeB:
            barrier -= mu * log( self.nlp.Uvar[i] - x[i] )
        return barrier

    # Evaluate the gradient of the barrier
    def GradBarrier( self, x, **kwargs ):
        mu = kwargs.get( 'mu', self.mu )

        g = self.nlp.grad(x)
        for i in self.nlp.lowerB + self.nlp.rangeB:
            g[i] -= mu * 1.0/( x[i] - self.nlp.Lvar[i] )
        for i in self.nlp.upperB + self.nlp.rangeB:
            g[i] += mu * 1.0/( self.nlp.Uvar[i] - x[i] )
        return g

    # Evaluate a product with the primal Hessian matrix
    def HessBarrierProd( self, x, p, **kwargs ):
        mu = kwargs.get( 'mu', self.mu )

        Hp = self.nlp.hprod( x, p )
        for i in self.nlp.lowerB + self.nlp.rangeB:
            Hp[i] += mu * 1.0/( x[i] - self.nlp.Lvar[i] )**2 * p[i]
        for i in self.nlp.lowerB + self.nlp.rangeB:
            Hp[i] += mu * 1.0/( self.nlp.Uvar[i] - x[i] )**2 * p[i]
        return Hp
        
    # Evaluate a product with the primal-dual Hessian matrix
    def PDHessBarrierProd( self, x, z, zR, p, **kwargs ):
        mu = kwargs.get( 'mu', self.mu )

        Hp = self.nlp.hprod( x, p )
        for i in self.nlp.lowerB:
            Hp[i] += 1.0/( x[i] - self.nlp.Lvar[i] ) * z[i] * p[i]
        for i in self.nlp.lowerB:
            Hp[i] += 1.0/( self.nlp.Uvar[i] - x[i] ) * z[i] * p[i]
        # two-sided bounds have two multipliers
        for i in self.nlp.rangeB:
            Hp[i] += 1.0/( x[i] - self.nlp.Lvar[i] ) * z[i] * p[i]
            Hp[i] += 1.0/( self.nlp.Uvar[i] - x[i] ) * zR[i] * p[i]
        return Hp
