# Some basic vector norms
from numpy import dot as Ndot
"""
Module norms implements various vector norms.
Vectors are usually understood as NumPy arrays
but might also be simply lists or tuples.

Contents

   norm1( x )      returns the 1-norm of x

   norm2( x )      returns the Euclidian norm of vector x

   normp( x, p )   returns the p-norm of x
                     norm1( x ) = norm( x, 1 )
                     norm2( x ) = norm( x, 2 )

   norm_infty( x ) returns the infinity (max) norm of x

$Id:$
"""
import math

def norm2( x ):
    "Returns the 2-norm (or Euclidian norm) of x"
    #return math.sqrt( sum( x*x ) )
    return math.sqrt( Ndot(x,x) )

def norm_infty( x ):
    "Returns the infinity (or max) norm of x"
    return max( abs( x ) )

def norm1( x ):
    "Returns the 1-norm of x"
    return sum( abs( x ) )

def normp( x, p ):
    "Returns the p-norm of x"
    # ( Sum |x_i|^p )^(1/p)
    # In fact, norm1(x) = norm(x,1)
    #          norm2(x) = norm(x,2)
    if int(p) == p and p > 0:
        return math.pow( sum( map( lambda y: math.pow(y,p), abs(x) ), 1/float(p) ) )
    return None
